#pragma inline
/*
 * this is a UNIX-style ioctl routine, but it can also
 * handle standard TurboC arguments, so it can replace Borland's ioctl.
 * Note that legal ioctl functions are small integers, so if
 * arg2 is less than 256, we will use TurboC conventions.
 * If bits 12-15 are all ones, then bits 8-11 are a generic ioctl for
 * handles device category code, and bits 0-7 are the minor function code.
 * Else if bits 13-15 are all ones, then it is a generic ioctl for block
 * devices ioctl.
 * Else if bits 8-15 are nonzero, then they are a standard DOS ioctl
 * code (range 00H-DFH), and bits 0-7 are placed in CX (range 0-255).
 * For example, in ASPI, to read the entry address, define GETASPI
 * as 0x204 (ioctlread<<8 | 4 bytes), and make the call
 *	ioctl(aspi, GETASPI, &buf).
 * On return, we get -1 and errno set on errors, and either 0 or data
 * on success.
 */
#define __IN_IOCTL	/* prevent function prototype errors */
#include <io.h>

int  pascal near __IOerror	  (int  __doserror);

int
ioctl(fd,func,argdx,cnt) int fd; int func; void *argdx; int cnt; {
	int argcx = cnt;	/* make a copy */
	int ax, dx;
	void far *argptr = argdx;

	/* massage input args */
	if ((func & 0xf000) == 0xf000) {	/* handle generic */
		argcx = func & 0xfff;
		func = 0xc;
	} else if ((func & 0xf000) == 0xe000) {	/* block generic */
		argcx = func & 0xfff;
		func = 0xd;
	} else if ((func & 0xff00) != 0) {	/* packed DOS */
		argcx = func & 255;
		func = (func >> 8) & 255;
	}

	/* call DOS */
	asm push ds;
	asm mov ah,44h;
	asm mov al,func;
	asm mov bx,fd;
	asm mov cx,argcx;
	asm lds dx,argptr;
	asm int 21h;
	asm pop ds;
	asm jc ioctlFailed;

	/* massage the results */
	ax = _AX;
	dx = _DX;
	switch (func) {
	default:	/* AX has data to be returned */
		return(ax);
	case 0: case 9: case 10:	/* DX has data to be returned */
		return(dx);
	case 1: case 11: case 12: case 13: case 16: case 17:	/* no data */
		return(0);
	case 6: case 7: case 14: case 15:	/* AL has data to be returned */
		return(ax&0xff);
	}

ioctlFailed:
	return	__IOerror (_AX);	/* returns -1 in AX on error */
}

#ifdef DOS5
/*
 * DOS 5 query ioctl command: packed generic only
 */
int
queryioctl(int fd, int func) {
	int argcx;
	extern errno;

	/* massage input args */
	if ((func & 0xf000) == 0xf000) {	/* handle generic */
		argcx = func & 0xfff;
		func = 0x10;
	} else  {				/* block generic */
		argcx = func & 0xfff;
		func = 0x11;
	}

	/* call dos */
	asm mov bx,fd;
	asm mov cx,argcx;
	asm mov al,func;
	asm mov ah,44h;
	asm int 21h;
	asm jc qioctlFailed;

	/* return 0 on success */
	return(0);
qioctlFailed:
	return	__IOerror (_AX);	/* returns -1 in AX on error */
}
#endif DOS5
